@@ -101,17 +101,20 @@ void RSGImage_by_Image_Tensor(float* inImage_Tensor,
 							  unsigned char optIters,
 						      float minimumResidual);
 
-void fitBackgroundRadially(float* inImage, unsigned char* inMask, 
-						   float* modelParamsMap, float* vecMP,
- 						   unsigned int minRes, 
-						   unsigned int maxRes, 
-						   unsigned int shellWidth,
-						   unsigned char includeCenter, 
-						   unsigned int finiteSampleBias,
-						   unsigned int X, unsigned int Y,
-						   float topkPerc, float botkPerc, 
-						   float MSSE_LAMBDA, 
-						   unsigned char optIters,
+void fitBackgroundRadially(float* inImage, unsigned char* inMask,
+                           float* modelParamsMap, float* vecMP,
+                           unsigned int minRes,
+                           unsigned int maxRes,
+                           unsigned int shellWidth,
+						   unsigned int stride,
+						   unsigned int X_Cent,
+						   unsigned int Y_Cent,
+                           unsigned char includeCenter,
+                           unsigned int finiteSampleBias,
+                           unsigned int X, unsigned int Y,
+                           float topkPerc, float botkPerc,
+                           float MSSE_LAMBDA,
+                           unsigned char optIters,
 						   float minimumResidual);
 
 void fitBackgroundCylindrically(float* inTensor,
