@@ -275,9 +275,21 @@ def sGHist_multi_mP(inVec, mP, SNR=3.0):
     plt.bar(modelVec, np.ones(modelVec.size), color='g',alpha=0.5)
     plt.show()
 
-def scatter3(mat, inFigure = None, returnFigure = False):
+def scatter3(mat, inFigure = None, returnFigure = False, label = None):
     """ given a matrix input of size 3 x N, it scatters it in 3D
     """
+    LWidth = 3
+    font = {
+            'weight' : 'bold',
+            'size'   : 8}
+    params = {'legend.fontsize': 'x-large',
+             'axes.labelsize': 'x-large',
+             'axes.titlesize':'x-large',
+             'xtick.labelsize':'x-large',
+             'ytick.labelsize':'x-large'}
+    plt.rc('font', **font)
+    plt.rcParams.update(params)
+
     if(inFigure is None):
         fig = plt.figure()
         ax = fig.add_subplot(111, projection='3d')
@@ -285,13 +297,72 @@ def scatter3(mat, inFigure = None, returnFigure = False):
     else:
         fig, ax, figureCnt = inFigure
         figureCnt += 1
-    ax.scatter(mat[0], mat[1], mat[2], label = 'Plot ' + str(figureCnt))
+    if(label is None):
+        label = 'Plot ' + str(figureCnt)
+    ax.scatter(mat[0], mat[1], mat[2], label = label)
     if(returnFigure):
         return((fig, ax, figureCnt))
     else:
         if(figureCnt>0):
             plt.legend()
         plt.show()
+
+class plotGaussianGradient:
+    def __init__(self, xlabel, ylabel, num_bars = 100, 
+                 title = None, xmin = None, xmax = None, 
+                 ymin = None, ymax = None):
+        self.xlabel = xlabel
+        self.ylabel = ylabel
+        self.title = title
+        self.num_bars = num_bars
+        self.xmin = xmin
+        self.xmax = xmax
+        self.ymin = ymin
+        self.ymax = ymax
+        LWidth = 1
+        font = {
+                'weight' : 'bold',
+                'size'   : 14}
+        plt.rc('font', **font)
+        params = {'legend.fontsize': 'x-large',
+                 'axes.labelsize': 'x-large',
+                 'axes.titlesize':'x-large',
+                 'xtick.labelsize':'x-large',
+                 'ytick.labelsize':'x-large'}
+        plt.rcParams.update(params)
+        plt.figure(figsize=(8, 6), dpi=50)
+        self.ax1 = plt.subplot(111)
+
+    def addPlot(self, x, mu, std, gradient_color, label, snr = 3.0):
+
+        for idx in range(self.num_bars-1):
+            y1 = ((self.num_bars-idx)*mu + idx*(mu + snr*std))/self.num_bars
+            y2 = y1 + snr*std/self.num_bars
+            
+            prob = np.exp(-(snr*idx/self.num_bars)**2/2)
+            plt.fill_between(
+                x, y1, y2, color = gradient_color, alpha = prob  )
+
+            y1 = ((self.num_bars-idx)*mu + idx*(mu - snr*std))/self.num_bars
+            y2 = y1 - snr*std/self.num_bars
+            
+            plt.fill_between(
+                x, y1, y2, color = gradient_color, alpha = prob  )
+        plt.plot(x, mu, linewidth = 3, color = gradient_color, label = label)
+        
+    def show(self, show_legend = True):
+        if(self.xmin is not None) & (self.xmax is not None):
+            plt.xlim([self.xmin, self.xmax])
+        if(self.ymin is not None) & (self.ymax is not None):
+            plt.ylim([self.ymin, self.ymax])
+        plt.xlabel(self.xlabel)
+        plt.ylabel(self.ylabel)
+        if(self.title is not None):
+            plt.title(self.title)
+        if(show_legend):
+            plt.legend()
+        plt.tight_layout()
+        plt.show()
     
 def getTriangularVertices(n,
                           rotationAngles = [0, 0, 0],
@@ -337,17 +408,45 @@ def getTriangularVertices(n,
 class multiprocessor:
     """ multiprocessor makes the use of multiprocessing in Python easy
     You need a function that can interpret the input arguments. Then you
-    need to make the inputs argument to be an indexable list of tuples.
+    need to make the inputs argument to be an indexable object by inputes[0]
+    to inputs[N].
     These N tuples each, will be the input given to a processor, which
     is the function you provide. The function provides an output and it
     will be given as a list of the ourputs with the same order as the input.
     You need to use the list as the list of outputs associated with the list of
     the inputs.
+    parameters: they will not be indexed but will be sent to the function
+        without change.
+        
+    A code snippet is brought here for the case of non numpy input output
+    
+    def multiprocessor_targetFunc(tuple_of_indexables, tuple_of_nonindexables):
+        op_type = tuple_of_nonindexables
+        data, mask = tuple_of_indexables
+        if(op_type=='median'):
+            to_return = np.median(data[mask==1])
+        return(np.array([to_return]))
+        
+    def test_multiprocessor():
+        N = 1000
+        D = 100000
+        Data = (10+100*np.random.randn(N,D)).astype('int')
+        Mask = (2*np.random.rand(N,D)).astype('int')
+        Param = 'median'
+        
+        tuple_of_indexables = (Data, Mask)
+        tuple_of_nonindexables = (Param) 
+        medians = RobustGaussianFittingLibrary.misc.multiprocessor(
+            multiprocessor_targetFunc, 
+            tuple_of_indexables, tuple_of_nonindexables).start()
+        
     """    
     def __init__(self, 
         targetFunction,
-        inputs,
+        inputs, 
+        parameters = None,
         max_cpu = None,
+        batchSize = None,
         showProgress = False):
         """
         input arguments
@@ -357,97 +456,176 @@ class multiprocessor:
                 the targetFunction, such as by syntax:
                     targetFunction(inputs[0])
                 Indeed we actually do this syntax before anything. 
+            parameters: they will not be indexed but will be sent to the function
+                without change. default = None
             max_cpu: max number of allowed cpu
                 default: None
+            batchSize: how many data points are sent to each CPU at a time
+                default: n_CPU/n_points/2
             showProgress: using textProgBar, it shows the progress of 
                 multiprocessing of your task.
                 default: False
         """
-        try:
-            funcOutput = targetFunction(inputs[0])
-        except:
-            print('Running the following syntax raised an exception:')
-            print('targetFunction(inputs[0])')
-            print('I cannot call your function with input[0]')
-            print('You need to make inputs mutable and indexable by axis 0')
-            exit()
-
         if(type(inputs).__module__ == np.__name__):
-            self.n_pts = inputs.shape[0]
+            self.inputIsNumpy = True
+            self.n_pts = inputs.shape[0]     
             if(showProgress):
-                print('Input is a numpy ndArray')
+                print('Input is a numpy ndArray with ', 
+                      self.n_pts, ' data points')
         else:
+            self.inputIsNumpy = False
             try:
-                self.n_pts = len(inputs)
+                self.n_individualInputs = len(inputs)
+                self.n_pts = inputs[0].shape[0]
                 if(showProgress):
-                    print('Input is not a numpy ndArray')
+                    print('Input is not a numpy ndArray, yet with ', 
+                          self.n_pts, ' data points')
             except:
                 print('I can not get the length of the input list')
-                print('we are supporting lists, tuples and numpy ndarrays')
+                print('we are supporting list or tuple of indexable objects')
                 exit()
 
+        if(not self.inputIsNumpy):
+            inputs_first_tst = []
+            for memberCnt in range(self.n_individualInputs):
+                inputs_first_tst.append(inputs[memberCnt][0])
+        else:
+            inputs_first_tst = inputs[0]
+            
+        try:
+            if(parameters is not None):
+                funcOutput = targetFunction(inputs_first_tst, parameters)
+            else:
+                funcOutput = targetFunction(inputs_first_tst)
+        except:
+            print('I cannot call your function')
+            print('Running the following syntax raised an exception:')
+            if(parameters is not None):
+                print('targetFunction(inputs[0], parameters)')
+            else:
+                print('targetFunction(inputs[0])')
+            print('You need to make inputs mutable and indexable by axis 0')
+            exit()
+        if(showProgress):
+            print('I could call your given function with first data point')
+
+        self.outputIsNumpy = False
         if(type(funcOutput).__module__ == np.__name__):
             self.outputIsNumpy = True
+            self.output_shape = funcOutput.shape
+            self.output_dtype = funcOutput.dtype
+            self.allResults = np.zeros(
+                shape = ((self.n_pts,) + self.output_shape), 
+                dtype = self.output_dtype)
             if(showProgress):
                 print('output is a numpy ndArray')
+                print('output_shape, ', self.output_shape)
+                print('shape to prepare: ', ((self.n_pts,) + self.output_shape))
+                print('allResults shape, ', self.allResults.shape)
+        else:
+            self.n_individualOutputs = len(funcOutput)
+            
+            self.allResults = []
+            self.Q_procID = np.array([])
+            if(showProgress):
+                print('output is a list with ', 
+                      self.n_individualOutputs, ' members.')
+                output_types = []
+            
         self.targetFunction = targetFunction
         self.inputs = inputs
+        self.parameters = parameters
         self.showProgress = showProgress
-        self.max_cpu = max_cpu
-    
-    def _multiprocFunc(self, aQ, _procID, inputArgs):
-        result = self.targetFunction(inputArgs)
-        aQ.put(list([_procID, result]))
-
+        if(max_cpu is not None):
+            self.max_cpu = max_cpu
+        else:
+            self.max_cpu = cpu_count()
+        self.default_batchSize = int(np.ceil(self.n_pts/self.max_cpu/2))
+        if(batchSize is not None):
+            self.default_batchSize = batchSize
+        if(showProgress):
+            print('RGFLib multiprocessor initialized with:') 
+            print('max_cpu: ', self.max_cpu)
+            print('n_pts: ', self.n_pts)
+            print('default_batchSize: ', self.default_batchSize)
+        
+    def _multiprocFunc(self, theQ, procID_range):
+        if(self.outputIsNumpy):
+            allResults = np.zeros(
+                shape = ((procID_range.shape[0],) + self.output_shape), 
+                dtype = self.output_dtype)
+        else:
+            allResults = []
+        for idx, procCnt in enumerate(procID_range):
+            if(not self.inputIsNumpy):
+                inputs_single = []
+                for memberCnt in range(self.n_individualInputs):
+                    inputs_single.append(self.inputs[memberCnt][procCnt])
+            else:
+                inputs_single = self.inputs[procCnt]
+            
+            if(self.parameters is not None):
+                results = self.targetFunction(inputs_single, self.parameters)
+            else:
+                results = self.targetFunction(inputs_single)
+            if(self.outputIsNumpy):
+                allResults[idx] = results
+            else:
+                allResults.append(results)
+                
+        theQ.put(list([procID_range, allResults]))
+        
     def start(self):        
-        myCPUCount = cpu_count()-1
-        if(self.max_cpu is not None):
-            myCPUCount = np.minimum(self.max_cpu, myCPUCount)
         aQ = Queue()
         numProc = self.n_pts
         procID = 0
         numProcessed = 0
         numBusyCores = 0
-        if(not self.outputIsNumpy):
-            allResults = []
-        Q_procID = np.zeros(self.n_pts, dtype='uint')
+        firstProcess = True
         while(numProcessed<numProc):
             if (not aQ.empty()):
                 aQElement = aQ.get()
-                Q_procID[numProcessed] = aQElement[0]
-                if(not self.outputIsNumpy):
-                    allResults.append(aQElement[1])
+                _batchSize = aQElement[0].shape[0]
+                if(self.outputIsNumpy):
+                    self.allResults[aQElement[0]] = aQElement[1]
                 else:
-                    if(numProcessed == 0):
-                        allResults = np.zeros(
-                            shape = ((self.n_pts,) + aQElement[1].shape), 
-                            dtype = aQElement[1].dtype)
-                    allResults[aQElement[0]] = aQElement[1] 
-                numProcessed += 1
+                    self.Q_procID = np.concatenate((self.Q_procID, aQElement[0]))
+                    self.allResults += aQElement[1]
+                numProcessed += _batchSize
                 numBusyCores -= 1
                 if(self.showProgress):
-                    if(numProcessed == 1):
+                    if(firstProcess):
                         pBar = textProgBar(numProc-1, title = 'starting ' \
                             + str(numProc) + ' processes with ' \
-                            + str(myCPUCount) + ' CPUs')
-                    if(numProcessed > 1):
-                        pBar.go()
+                            + str(self.max_cpu) + ' CPUs')
+                        firstProcess = False
+                    else:
+                        pBar.go(_batchSize)
     
-            if((procID<numProc) & (numBusyCores < myCPUCount)):
-                Process(target = self._multiprocFunc, args = (aQ, procID, 
-                                           self.inputs[procID])).start()
-                procID += 1
+            if((procID<numProc) & (numBusyCores < self.max_cpu)):
+                batchSize = np.minimum(self.default_batchSize, numProc - procID)
+                procID_arange = np.arange(procID, procID + batchSize, dtype = 'int')
+                Process(target = self._multiprocFunc, 
+                        args = (aQ, procID_arange)).start()
+                procID += batchSize
                 numBusyCores += 1
         if(self.showProgress):
             del pBar
         
-        if(not self.outputIsNumpy):
-            sortInds = np.argsort(Q_procID)
-            _output = []
-            for cnt in sortInds:
-                _output.append(allResults[cnt])
-            return (_output)
+        if(self.outputIsNumpy):
+            return (self.allResults)
         else:
-            return (allResults)
-
+            sortArgs = np.argsort(self.Q_procID)
+            ret_list = [self.allResults[i] for i in sortArgs]
+            endResultList = []
+            for memberCnt in range(self.n_individualOutputs):
+                _currentList = []
+                for ptCnt in range(self.n_pts):
+                    _currentList.append(ret_list[ptCnt][memberCnt])
+                endResultList.append(_currentList)
+            for memberCnt in range(self.n_individualOutputs):
+                if(type(endResultList[memberCnt][0]).__module__ == np.__name__):
+                    endResultList[memberCnt] = \
+                        np.vstack(endResultList[memberCnt])
+            return (endResultList)
 
